# API 架构重构说明

## 概述

本次重构将原有的网络请求代码进行了全面的重新设计和组织，提供了更加模块化、类型安全和易于扩展的API架构。

## 目录结构

```
src/api/
├── types/           # 类型定义
│   └── index.ts
├── config/          # 配置文件
│   └── index.ts
├── utils/           # 工具函数
│   └── index.ts
├── core/            # 核心模块
│   ├── request.ts   # HTTP请求客户端
│   ├── upload.ts    # 文件上传模块
│   └── download.ts  # 文件下载模块
├── modules/         # 业务API模块
│   ├── editor.ts    # 编辑器相关API
│   └── resource.ts  # 资源管理API
├── examples/        # 使用示例
│   └── usage.ts
├── index.ts         # 统一入口
└── README.md        # 说明文档
```

## 主要特性

### 1. 类型安全
- 完整的TypeScript类型定义
- 泛型支持，确保请求和响应数据的类型安全
- 严格的类型检查

### 2. 模块化设计
- 核心功能与业务逻辑分离
- 可插拔的模块设计
- 易于扩展和维护

### 3. 文件上传功能
- 普通文件上传
- 图片上传（支持压缩）
- 大文件分片上传
- 批量文件上传
- Base64上传
- 上传进度监控

### 4. 文件下载功能
- 普通文件下载
- 分片下载（大文件）
- 批量下载
- 下载为不同格式（Blob、Base64、ArrayBuffer、Text）
- 下载进度监控

### 5. 错误处理
- 统一的错误处理机制
- 自动重试功能
- 详细的错误信息

### 6. 请求管理
- 请求拦截器
- 响应拦截器
- 请求取消功能
- 认证token管理

## 快速开始

### 基本使用

```typescript
import { editorApi, resourceApi, ApiManager } from '@/api'

// 设置认证token
ApiManager.setAuthToken('your-token')

// 创建订单
const order = await editorApi.createOrder({
  projectId: 'project-123',
  type: 'render'
})

// 上传资源
const resource = await resourceApi.uploadResource(file, {
  type: 'model',
  tags: ['建筑', '现代']
})
```

### 文件上传

```typescript
import { fileUploader } from '@/api'

// 普通文件上传
const result = await fileUploader.uploadFile({
  file,
  onProgress: (progress) => {
    console.log(`上传进度: ${progress}%`)
  }
})

// 图片上传（带压缩）
const imageResult = await fileUploader.uploadImage(file, {
  compress: true,
  quality: 0.8,
  maxWidth: 1920,
  maxHeight: 1080
})

// 大文件分片上传
const largeFileResult = await fileUploader.uploadLargeFile(file, {
  chunkSize: 2 * 1024 * 1024, // 2MB per chunk
  onProgress: (progress) => {
    console.log(`上传进度: ${progress}%`)
  }
})
```

### 文件下载

```typescript
import { fileDownloader } from '@/api'

// 下载并保存文件
await fileDownloader.downloadAndSave({
  url: 'https://example.com/file.zip',
  filename: 'download.zip'
})

// 下载为Base64
const base64 = await fileDownloader.downloadAsBase64({
  url: 'https://example.com/image.jpg'
})

// 批量下载
const result = await fileDownloader.downloadBatch([
  'https://example.com/file1.zip',
  'https://example.com/file2.zip'
], {
  concurrent: 3,
  onProgress: (progress) => {
    console.log(`批量下载进度: ${progress}%`)
  }
})
```

## API 模块

### 编辑器API (EditorApi)

提供编辑器相关的功能：

- `createOrder(params)` - 创建订单
- `saveProject(projectData)` - 保存项目
- `loadProject(projectId)` - 加载项目
- `deleteProject(projectId)` - 删除项目
- `getProjectList(params)` - 获取项目列表
- `uploadThumbnail(file, projectId)` - 上传缩略图
- `exportProject(projectId, format)` - 导出项目
- `importProject(file)` - 导入项目

### 资源API (ResourceApi)

提供资源管理功能：

- `getResourceList(params)` - 获取资源列表
- `getResourceDetail(resourceId)` - 获取资源详情
- `uploadResource(file, metadata)` - 上传资源
- `uploadMultipleResources(files, metadata)` - 批量上传资源
- `deleteResource(resourceId)` - 删除资源
- `downloadResource(resource)` - 下载资源
- `searchResources(params)` - 搜索资源

## 配置

### 环境变量

```bash
# .env.development
VITE_BASE_URL=http://localhost:3000/api

# .env.production
VITE_BASE_URL=https://api.example.com
```

### API配置

```typescript
// src/api/config/index.ts
export const API_CONFIG = {
  BASE_URL: import.meta.env.VITE_BASE_URL,
  TIMEOUT: 15000,
  RETRY_COUNT: 3,
  RETRY_DELAY: 1000,
  // ...其他配置
}
```

## 错误处理

API会自动处理常见的错误情况：

- 网络连接失败
- 请求超时
- 认证失败
- 权限不足
- 服务器错误

所有API方法都返回统一的响应格式：

```typescript
interface ApiResponse<T> {
  code: number
  message: string
  data: T
  success: boolean
}
```

## 扩展性

### 添加新的API模块

1. 在 `src/api/modules/` 目录下创建新的模块文件
2. 定义相关的类型接口
3. 实现API类
4. 在 `src/api/index.ts` 中导出

### 添加新的请求拦截器

```typescript
import { httpClient } from '@/api'

// 添加请求拦截器
httpClient.instance.interceptors.request.use(
  (config) => {
    // 自定义请求处理
    return config
  },
  (error) => {
    return Promise.reject(error)
  }
)
```

## 迁移指南

### 从旧API迁移

旧的API调用：
```typescript
// 旧方式
const request = new Request()
request.post(new ParamsReq('/api/endpoint', data))
```

新的API调用：
```typescript
// 新方式
import { httpClient } from '@/api'
const response = await httpClient.post('/api/endpoint', data)
```

### 兼容性

新的API架构与旧代码完全分离，可以逐步迁移，不会影响现有功能。

## 最佳实践

1. **使用TypeScript类型**：充分利用类型定义，确保代码的类型安全
2. **错误处理**：始终处理API调用可能出现的错误
3. **进度监控**：对于文件上传下载，使用进度回调提升用户体验
4. **请求取消**：在组件卸载时取消未完成的请求
5. **缓存策略**：对于不经常变化的数据，考虑实现缓存机制

## 性能优化

1. **并发控制**：批量操作时控制并发数量
2. **分片上传**：大文件使用分片上传
3. **压缩优化**：图片上传时自动压缩
4. **请求去重**：避免重复请求
5. **缓存机制**：合理使用缓存减少网络请求

## 示例代码

详细的使用示例请参考 `src/api/examples/usage.ts` 文件。 