import { httpClient } from './request'
import { DownloadConfig, ApiResponse } from '../types'
import { API_ENDPOINTS } from '../config'
import { downloadFile, generateId } from '../utils'

/**
 * 下载响应类型
 */
export interface DownloadResponse {
  blob: Blob
  filename: string
  size: number
  type: string
}

/**
 * 批量下载响应类型
 */
export interface BatchDownloadResponse {
  success: DownloadResponse[]
  failed: Array<{
    url: string
    error: string
  }>
  total: number
  successCount: number
  failedCount: number
}

/**
 * 文件下载类
 */
export class FileDownloader {
  private downloadQueue: Map<string, AbortController> = new Map()

  /**
   * 下载文件
   */
  public async downloadFile(config: DownloadConfig): Promise<DownloadResponse> {
    const {
      url,
      filename,
      params,
      onProgress
    } = config

    // 生成下载ID
    const downloadId = generateId()
    const controller = new AbortController()
    this.downloadQueue.set(downloadId, controller)

    try {
      const response = await httpClient.request<Blob>({
        url,
        method: 'GET',
        params,
        responseType: 'blob',
        timeout: 0, // 下载不设置超时
      })

      if (!response.success) {
        throw new Error(response.message)
      }

      const blob = response.data
      const contentType = blob.type || 'application/octet-stream'
      
      // 从响应头或URL中获取文件名
      const finalFilename = filename || this.extractFilenameFromUrl(url) || 'download'

      return {
        blob,
        filename: finalFilename,
        size: blob.size,
        type: contentType
      }
    } finally {
      this.downloadQueue.delete(downloadId)
    }
  }

  /**
   * 下载并保存文件
   */
  public async downloadAndSave(config: DownloadConfig): Promise<void> {
    const result = await this.downloadFile(config)
    downloadFile(result.blob, result.filename)
  }

  /**
   * 下载文件为Base64
   */
  public async downloadAsBase64(config: DownloadConfig): Promise<string> {
    const result = await this.downloadFile(config)
    
    return new Promise((resolve, reject) => {
      const reader = new FileReader()
      reader.onload = () => {
        const base64 = reader.result as string
        resolve(base64.split(',')[1]) // 移除data:xxx;base64,前缀
      }
      reader.onerror = reject
      reader.readAsDataURL(result.blob)
    })
  }

  /**
   * 下载文件为ArrayBuffer
   */
  public async downloadAsArrayBuffer(config: DownloadConfig): Promise<ArrayBuffer> {
    const result = await this.downloadFile(config)
    
    return new Promise((resolve, reject) => {
      const reader = new FileReader()
      reader.onload = () => resolve(reader.result as ArrayBuffer)
      reader.onerror = reject
      reader.readAsArrayBuffer(result.blob)
    })
  }

  /**
   * 下载文件为文本
   */
  public async downloadAsText(config: DownloadConfig): Promise<string> {
    const result = await this.downloadFile(config)
    
    return new Promise((resolve, reject) => {
      const reader = new FileReader()
      reader.onload = () => resolve(reader.result as string)
      reader.onerror = reject
      reader.readAsText(result.blob)
    })
  }

  /**
   * 分片下载（大文件）
   */
  public async downloadLargeFile(
    config: DownloadConfig & {
      chunkSize?: number
      onChunkProgress?: (chunkIndex: number, chunkProgress: number) => void
    }
  ): Promise<DownloadResponse> {
    const {
      url,
      filename,
      params,
      chunkSize = 2 * 1024 * 1024, // 2MB per chunk
      onProgress,
      onChunkProgress
    } = config

    try {
      // 获取文件信息
      const headResponse = await httpClient.request({
        url,
        method: 'HEAD',
        params
      })

      if (!headResponse.success) {
        throw new Error('无法获取文件信息')
      }

      // 这里需要根据实际的响应头来获取文件大小
      // const fileSize = parseInt(headResponse.headers['content-length'] || '0')
      const fileSize = 0 // 临时设置，实际应该从响应头获取

      if (fileSize === 0) {
        // 如果无法获取文件大小，回退到普通下载
        return this.downloadFile(config)
      }

      const totalChunks = Math.ceil(fileSize / chunkSize)
      const chunks: Blob[] = []

      // 下载分片
      for (let i = 0; i < totalChunks; i++) {
        const start = i * chunkSize
        const end = Math.min(start + chunkSize - 1, fileSize - 1)

        const chunkResponse = await httpClient.request<Blob>({
          url,
          method: 'GET',
          params,
          headers: {
            'Range': `bytes=${start}-${end}`
          },
          responseType: 'blob'
        })

        if (chunkResponse.success) {
          chunks.push(chunkResponse.data)
          
          // 分片进度回调
          onChunkProgress?.(i, 100)
          
          // 总进度回调
          const progress = ((i + 1) / totalChunks) * 100
          onProgress?.(progress)
        } else {
          throw new Error(`分片 ${i} 下载失败: ${chunkResponse.message}`)
        }
      }

      // 合并分片
      const blob = new Blob(chunks)
      const finalFilename = filename || this.extractFilenameFromUrl(url) || 'download'

      return {
        blob,
        filename: finalFilename,
        size: blob.size,
        type: blob.type || 'application/octet-stream'
      }
    } catch (error) {
      throw new Error(`分片下载失败: ${error}`)
    }
  }

  /**
   * 批量下载
   */
  public async downloadBatch(
    urls: string[],
    options: {
      concurrent?: number
      onProgress?: (progress: number) => void
      onFileProgress?: (url: string, progress: number) => void
      onFileComplete?: (url: string, result: DownloadResponse | Error) => void
    } = {}
  ): Promise<BatchDownloadResponse> {
    const {
      concurrent = 3,
      onProgress,
      onFileProgress,
      onFileComplete
    } = options

    const results: BatchDownloadResponse = {
      success: [],
      failed: [],
      total: urls.length,
      successCount: 0,
      failedCount: 0
    }

    const downloadUrl = async (url: string): Promise<void> => {
      try {
        const response = await this.downloadFile({
          url,
          onProgress: (progress) => onFileProgress?.(url, progress)
        })

        results.success.push(response)
        results.successCount++
        onFileComplete?.(url, response)
      } catch (error) {
        results.failed.push({
          url,
          error: error instanceof Error ? error.message : '下载失败'
        })
        results.failedCount++
        onFileComplete?.(url, error as Error)
      }

      // 更新总进度
      const completed = results.successCount + results.failedCount
      const progress = (completed / results.total) * 100
      onProgress?.(progress)
    }

    // 并发下载
    const chunks = []
    for (let i = 0; i < urls.length; i += concurrent) {
      chunks.push(urls.slice(i, i + concurrent))
    }

    for (const chunk of chunks) {
      await Promise.all(chunk.map(downloadUrl))
    }

    return results
  }

  /**
   * 批量下载并打包为ZIP
   */
  public async downloadAsZip(
    files: Array<{
      url: string
      filename?: string
    }>,
    zipFilename: string = 'download.zip'
  ): Promise<void> {
    try {
      // 下载所有文件
      const downloadPromises = files.map(async (file) => {
        const result = await this.downloadFile({
          url: file.url,
          filename: file.filename
        })
        return {
          filename: result.filename,
          blob: result.blob
        }
      })

      const downloadedFiles = await Promise.all(downloadPromises)

      // 这里需要使用JSZip库来创建ZIP文件
      // 由于没有导入JSZip，这里只是示例代码
      console.log('需要集成JSZip库来实现ZIP打包功能')
      console.log('下载的文件:', downloadedFiles)

      // 示例：如果有JSZip
      // const JSZip = (await import('jszip')).default
      // const zip = new JSZip()
      // 
      // downloadedFiles.forEach(file => {
      //   zip.file(file.filename, file.blob)
      // })
      // 
      // const zipBlob = await zip.generateAsync({ type: 'blob' })
      // downloadFile(zipBlob, zipFilename)

    } catch (error) {
      throw new Error(`ZIP下载失败: ${error}`)
    }
  }

  /**
   * 从URL中提取文件名
   */
  private extractFilenameFromUrl(url: string): string {
    try {
      const urlObj = new URL(url)
      const pathname = urlObj.pathname
      const filename = pathname.split('/').pop()
      return filename || 'download'
    } catch {
      return 'download'
    }
  }

  /**
   * 取消下载
   */
  public cancelDownload(downloadId: string): void {
    const controller = this.downloadQueue.get(downloadId)
    if (controller) {
      controller.abort()
      this.downloadQueue.delete(downloadId)
    }
  }

  /**
   * 取消所有下载
   */
  public cancelAllDownloads(): void {
    this.downloadQueue.forEach((controller) => {
      controller.abort()
    })
    this.downloadQueue.clear()
  }

  /**
   * 获取下载进度
   */
  public getDownloadProgress(downloadId: string): number {
    // 这里可以实现获取具体下载进度的逻辑
    return 0
  }
}

// 创建默认实例
export const fileDownloader = new FileDownloader() 