import axios, { AxiosInstance, AxiosRequestConfig, AxiosResponse, AxiosError } from 'axios'
import { ApiResponse, RequestConfig, ApiError } from '../types'
import { API_CONFIG, ENV_CONFIG } from '../config'
import { createApiError, retry } from '../utils'

/**
 * HTTP请求客户端类
 */
export class HttpClient {
  private instance: AxiosInstance
  private requestQueue: Map<string, AbortController> = new Map()

  constructor() {
    this.instance = this.createInstance()
    this.setupInterceptors()
  }

  /**
   * 创建axios实例
   */
  private createInstance(): AxiosInstance {
    return axios.create({
      baseURL: API_CONFIG.BASE_URL,
      timeout: API_CONFIG.TIMEOUT,
      headers: API_CONFIG.DEFAULT_HEADERS,
    })
  }

  /**
   * 设置拦截器
   */
  private setupInterceptors(): void {
    // 请求拦截器
    this.instance.interceptors.request.use(
      (config) => {
        // 添加请求ID用于取消请求
        const requestId = this.generateRequestId(config)
        config.metadata = { requestId }

        // 添加认证token
        const token = this.getAuthToken()
        if (token && config.headers) {
          config.headers.Authorization = `Bearer ${token}`
        }

        // 开发环境下打印请求信息
        if (ENV_CONFIG.isDev) {
          console.log('🚀 Request:', {
            url: config.url,
            method: config.method,
            params: config.params,
            data: config.data,
          })
        }

        return config
      },
      (error) => {
        return Promise.reject(this.handleError(error))
      }
    )

    // 响应拦截器
    this.instance.interceptors.response.use(
      (response) => {
        // 清理请求队列
        const requestId = response.config.metadata?.requestId
        if (requestId) {
          this.requestQueue.delete(requestId)
        }

        // 开发环境下打印响应信息
        if (ENV_CONFIG.isDev) {
          console.log('✅ Response:', {
            url: response.config.url,
            status: response.status,
            data: response.data,
          })
        }

        // 直接返回response，在具体的请求方法中处理数据格式
        return response
      },
      (error) => {
        // 清理请求队列
        const requestId = error.config?.metadata?.requestId
        if (requestId) {
          this.requestQueue.delete(requestId)
        }

        return Promise.reject(this.handleError(error))
      }
    )
  }

  /**
   * 处理响应数据
   */
  private handleResponse<T>(response: AxiosResponse): ApiResponse<T> {
    const { data } = response

    // 如果后端返回的数据已经是标准格式
    if (data && typeof data.code !== 'undefined') {
      return {
        code: data.code,
        message: data.message || 'Success',
        data: data.data,
        success: data.code === API_CONFIG.BUSINESS_CODES.SUCCESS,
      }
    }

    // 否则包装成标准格式
    return {
      code: API_CONFIG.BUSINESS_CODES.SUCCESS,
      message: 'Success',
      data: data,
      success: true,
    }
  }

  /**
   * 处理错误
   */
  private handleError(error: any): ApiError {
    if (ENV_CONFIG.isDev) {
      console.error('❌ Request Error:', error)
    }

    // 请求被取消
    if (axios.isCancel(error)) {
      return createApiError(-1, '请求已取消')
    }

    // 网络错误
    if (!error.response) {
      return createApiError(-1, '网络连接失败，请检查网络设置')
    }

    const { status, data } = error.response as AxiosResponse

    // 根据HTTP状态码处理
    switch (status) {
      case API_CONFIG.STATUS_CODES.UNAUTHORIZED:
        this.handleUnauthorized()
        return createApiError(status, '登录已过期，请重新登录')
      
      case API_CONFIG.STATUS_CODES.FORBIDDEN:
        return createApiError(status, '没有权限访问该资源')
      
      case API_CONFIG.STATUS_CODES.NOT_FOUND:
        return createApiError(status, '请求的资源不存在')
      
      case API_CONFIG.STATUS_CODES.SERVER_ERROR:
        return createApiError(status, '服务器内部错误')
      
      default:
        return createApiError(
          status,
          data?.message || `请求失败 (${status})`,
          data
        )
    }
  }

  /**
   * 处理未授权
   */
  private handleUnauthorized(): void {
    // 清除token
    this.clearAuthToken()
    
    // 可以在这里添加跳转到登录页的逻辑
    // router.push('/login')
  }

  /**
   * 生成请求ID
   */
  private generateRequestId(config: AxiosRequestConfig): string {
    return `${config.method}_${config.url}_${Date.now()}`
  }

  /**
   * 获取认证token
   */
  private getAuthToken(): string | null {
    return localStorage.getItem('auth_token')
  }

  /**
   * 清除认证token
   */
  private clearAuthToken(): void {
    localStorage.removeItem('auth_token')
  }

  /**
   * 设置认证token
   */
  public setAuthToken(token: string): void {
    localStorage.setItem('auth_token', token)
  }

  /**
   * 通用请求方法
   */
  public async request<T = any>(config: RequestConfig): Promise<ApiResponse<T>> {
    const axiosConfig: AxiosRequestConfig = {
      url: config.url,
      method: config.method || 'GET',
      params: config.params,
      data: config.data,
      headers: config.headers,
      timeout: config.timeout,
      responseType: config.responseType || 'json',
    }

    const response = await this.instance.request(axiosConfig)
    return this.handleResponse<T>(response)
  }

  /**
   * 带重试的请求方法
   */
  public async requestWithRetry<T = any>(
    config: RequestConfig,
    retries?: number
  ): Promise<ApiResponse<T>> {
    return retry(() => this.request<T>(config), retries)
  }

  /**
   * GET请求
   */
  public get<T = any>(
    url: string,
    params?: Record<string, any>,
    config?: Partial<RequestConfig>
  ): Promise<ApiResponse<T>> {
    return this.request<T>({
      url,
      method: 'GET',
      params,
      ...config,
    })
  }

  /**
   * POST请求
   */
  public post<T = any>(
    url: string,
    data?: any,
    config?: Partial<RequestConfig>
  ): Promise<ApiResponse<T>> {
    return this.request<T>({
      url,
      method: 'POST',
      data,
      ...config,
    })
  }

  /**
   * PUT请求
   */
  public put<T = any>(
    url: string,
    data?: any,
    config?: Partial<RequestConfig>
  ): Promise<ApiResponse<T>> {
    return this.request<T>({
      url,
      method: 'PUT',
      data,
      ...config,
    })
  }

  /**
   * DELETE请求
   */
  public delete<T = any>(
    url: string,
    params?: Record<string, any>,
    config?: Partial<RequestConfig>
  ): Promise<ApiResponse<T>> {
    return this.request<T>({
      url,
      method: 'DELETE',
      params,
      ...config,
    })
  }

  /**
   * PATCH请求
   */
  public patch<T = any>(
    url: string,
    data?: any,
    config?: Partial<RequestConfig>
  ): Promise<ApiResponse<T>> {
    return this.request<T>({
      url,
      method: 'PATCH',
      data,
      ...config,
    })
  }

  /**
   * 取消请求
   */
  public cancelRequest(requestId: string): void {
    const controller = this.requestQueue.get(requestId)
    if (controller) {
      controller.abort()
      this.requestQueue.delete(requestId)
    }
  }

  /**
   * 取消所有请求
   */
  public cancelAllRequests(): void {
    this.requestQueue.forEach((controller) => {
      controller.abort()
    })
    this.requestQueue.clear()
  }
}

// 创建默认实例
export const httpClient = new HttpClient()

// 扩展axios配置类型
declare module 'axios' {
  interface AxiosRequestConfig {
    metadata?: {
      requestId: string
    }
  }
} 