// 核心模块
export { httpClient, HttpClient } from './core/request'
export { fileUploader, FileUploader } from './core/upload'
export { fileDownloader, FileDownloader } from './core/download'

// 业务API模块
export { editorApi, EditorApi } from './modules/editor'
export { resourceApi, ResourceApi } from './modules/resource'

// 类型定义
export type {
  ApiResponse,
  RequestConfig,
  UploadConfig,
  DownloadConfig,
  PaginationParams,
  PaginationResponse,
  ApiError
} from './types'

// 配置
export { API_CONFIG, API_ENDPOINTS, ENV_CONFIG } from './config'

// 工具函数
export {
  delay,
  generateId,
  formatFileSize,
  getFileExtension,
  validateFileType,
  createApiError,
  isApiError,
  retry,
  downloadFile,
  readFileAsBase64,
  readFileAsArrayBuffer,
  compressImage
} from './utils'

// 业务类型
export type {
  ProjectData,
  OrderData
} from './modules/editor'

export type {
  ResourceData,
  ResourceCategory,
  ResourceSearchParams
} from './modules/resource'

// 上传下载相关类型
export type {
  UploadResponse,
  BatchUploadResponse
} from './core/upload'

export type {
  DownloadResponse,
  BatchDownloadResponse
} from './core/download'

// 导入实例用于ApiManager
import { httpClient } from './core/request'
import { fileUploader } from './core/upload'
import { fileDownloader } from './core/download'
import { API_CONFIG, ENV_CONFIG } from './config'

/**
 * API管理器类
 * 提供统一的API管理和配置
 */
export class ApiManager {
  /**
   * 设置认证token
   */
  public static setAuthToken(token: string): void {
    httpClient.setAuthToken(token)
  }

  /**
   * 取消所有请求
   */
  public static cancelAllRequests(): void {
    httpClient.cancelAllRequests()
    fileUploader.cancelAllUploads()
    fileDownloader.cancelAllDownloads()
  }

  /**
   * 获取API配置
   */
  public static getConfig() {
    return {
      baseUrl: API_CONFIG.BASE_URL,
      timeout: API_CONFIG.TIMEOUT,
      retryCount: API_CONFIG.RETRY_COUNT,
      env: ENV_CONFIG
    }
  }

  /**
   * 检查网络连接
   */
  public static async checkConnection(): Promise<boolean> {
    try {
      const response = await httpClient.get('/health')
      return response.success
    } catch {
      return false
    }
  }
}

// 默认导出API管理器
export default ApiManager 