/**
 * 应用状态管理
 */
import { defineStore } from 'pinia'
import { ref, computed } from 'vue'

export type Theme = 'light' | 'dark' | 'auto'
export type Language = 'zh-CN' | 'en-US'

export interface AppConfig {
  theme: Theme
  language: Language
  collapsed: boolean
  loading: boolean
  title: string
}

export const useAppStore = defineStore('app', () => {
  // 应用配置
  const config = ref<AppConfig>({
    theme: 'light',
    language: 'zh-CN',
    collapsed: false,
    loading: false,
    title: 'Vite应用'
  })

  // 侧边栏折叠状态
  const collapsed = computed(() => config.value.collapsed)
  
  // 主题
  const theme = computed(() => config.value.theme)
  
  // 语言
  const language = computed(() => config.value.language)
  
  // 加载状态
  const loading = computed(() => config.value.loading)
  
  // 应用标题
  const title = computed(() => config.value.title)

  // 设置主题
  const setTheme = (newTheme: Theme) => {
    config.value.theme = newTheme
    // 应用主题到文档
    document.documentElement.setAttribute('data-theme', newTheme)
  }

  // 切换主题
  const toggleTheme = () => {
    const newTheme = theme.value === 'light' ? 'dark' : 'light'
    setTheme(newTheme)
  }

  // 设置语言
  const setLanguage = (newLanguage: Language) => {
    config.value.language = newLanguage
  }

  // 切换侧边栏折叠状态
  const toggleCollapsed = () => {
    config.value.collapsed = !config.value.collapsed
  }

  // 设置侧边栏折叠状态
  const setCollapsed = (newCollapsed: boolean) => {
    config.value.collapsed = newCollapsed
  }

  // 设置加载状态
  const setLoading = (newLoading: boolean) => {
    config.value.loading = newLoading
  }

  // 设置应用标题
  const setTitle = (newTitle: string) => {
    config.value.title = newTitle
    document.title = newTitle
  }

  // 更新配置
  const updateConfig = (updates: Partial<AppConfig>) => {
    config.value = { ...config.value, ...updates }
  }

  // 重置配置
  const resetConfig = () => {
    config.value = {
      theme: 'light',
      language: 'zh-CN',
      collapsed: false,
      loading: false,
      title: 'Vite应用'
    }
  }

  // 初始化应用
  const initApp = () => {
    // 应用主题
    setTheme(theme.value)
    // 设置标题
    setTitle(title.value)
  }

  return {
    // 状态
    config,
    
    // 计算属性
    collapsed,
    theme,
    language,
    loading,
    title,
    
    // 方法
    setTheme,
    toggleTheme,
    setLanguage,
    toggleCollapsed,
    setCollapsed,
    setLoading,
    setTitle,
    updateConfig,
    resetConfig,
    initApp
  }
}) 