/**
 * 持久化存储工具
 */

export interface StorageOptions {
  prefix?: string
  storage?: Storage
  expire?: number
}

export class StorageUtil {
  private prefix: string
  private storage: Storage
  
  constructor(options: StorageOptions = {}) {
    this.prefix = options.prefix || 'app_'
    this.storage = options.storage || window.localStorage
  }

  /**
   * 生成存储key
   */
  private getKey(key: string): string {
    return `${this.prefix}${key}`
  }

  /**
   * 设置存储
   */
  set<T = any>(key: string, value: T, expire?: number): void {
    const storageData = {
      value,
      expire: expire ? Date.now() + expire * 1000 : null
    }
    this.storage.setItem(this.getKey(key), JSON.stringify(storageData))
  }

  /**
   * 获取存储
   */
  get<T = any>(key: string): T | null {
    const item = this.storage.getItem(this.getKey(key))
    if (!item) return null

    try {
      const storageData = JSON.parse(item)
      
      // 检查是否过期
      if (storageData.expire && Date.now() > storageData.expire) {
        this.remove(key)
        return null
      }
      
      return storageData.value
    } catch (error) {
      console.error('Storage parse error:', error)
      return null
    }
  }

  /**
   * 删除存储
   */
  remove(key: string): void {
    this.storage.removeItem(this.getKey(key))
  }

  /**
   * 清空所有存储
   */
  clear(): void {
    const keys = Object.keys(this.storage)
    keys.forEach(key => {
      if (key.startsWith(this.prefix)) {
        this.storage.removeItem(key)
      }
    })
  }

  /**
   * 获取所有存储的keys
   */
  keys(): string[] {
    const keys = Object.keys(this.storage)
    return keys
      .filter(key => key.startsWith(this.prefix))
      .map(key => key.replace(this.prefix, ''))
  }

  /**
   * 检查是否存在
   */
  has(key: string): boolean {
    return this.get(key) !== null
  }
}

// 默认实例
export const localStorage = new StorageUtil({
  prefix: 'local_',
  storage: window.localStorage
})

export const sessionStorage = new StorageUtil({
  prefix: 'session_',
  storage: window.sessionStorage
})

// 便捷方法
export const storage = {
  local: localStorage,
  session: sessionStorage,
  
  // 创建自定义存储实例
  create: (options: StorageOptions) => new StorageUtil(options)
} 